using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Drawing;
using System.Globalization;
using System.Threading;
using WealthLab;
using WealthLab.Indicators;
using WealthLab.Rules;

namespace WealthLab.Strategies
{
	public class MyStrategy : WealthScript
	{				
		private StrategyParameter paramThreshold;
		private StrategyParameter paramDecline;
		private StrategyParameter paramExitAfter;
		private StrategyParameter paramTimeoutEntry;
						
		public MyStrategy()
		{
			paramThreshold = CreateParameter("Threshold $", 300000, 500, 5000000, 500);     
			paramDecline = CreateParameter("Decline %", 50, 1, 85, 1);
			paramExitAfter = CreateParameter("Exit after, days", 15, 5, 60, 1);
			paramTimeoutEntry = CreateParameter("Timeout Entry", 62, 1, 62, 1);
		}
		
		protected override void Execute()
		{
			//Setup the debugging flag (show or not show)
			bool showDebug = false;
			
			//Setup the dataset switch flag
			string datasetFlag = "OI";
			
			// Illustrates the usage of FormatValue using the fundamental data item "insider transaction"

			const char tab = '\u0009';
			
			//OpenInsider Insider Transaction Data (10 years)
			
			IList<FundamentalItem> fListOI = null;
			
			if(datasetFlag == "OI" || showDebug == true)
			{
				string itemInsiderOpen = "[oi] insider transaction";
				fListOI = FundamentalDataItems(itemInsiderOpen);
			}

			//Clear Debug window for new records...
			//ClearDebug();
			
			if(showDebug == true)
			{				
				//Block of code for OpenInsider
				//IList<FundamentalItem> fListOI = FundamentalDataItems(itemInsiderOpen);            
				PrintDebug(Bars.Symbol + tab + "Item Count: " + fListOI.Count);
				PrintDebug("\n");
				PrintDebug("Insider Transaction (OpenInsider)");
				PrintDebug("\n");
				foreach (FundamentalItem fiOI in fListOI)
				{
					//Only looking for buy information...
					if(fiOI.Value >= 0)
					{
						PrintDebug(fiOI.FormatValue().Replace("\n", " "));
						PrintDebug(fiOI.Name);
						PrintDebug("Reporting Date: " + fiOI.Date.ToShortDateString());
						PrintDebug("Number of Shares: " + fiOI.Value.ToString());
						PrintDebug("Officer Name: " + fiOI.GetDetail("officer"));
						PrintDebug("Job Title: " + fiOI.GetDetail("relationship"));
						PrintDebug("Proceeds in Dollor Amount: " + fiOI.GetDetail("proceeds"));
						PrintDebug("Transaction Date: " + fiOI.GetDetail("psdate"));
						PrintDebug("\n");
					}
				}
			}
			
			DataSeries reverseAdjustment;
			DataSeriesOp.SplitReverseFactor( this, "split", out reverseAdjustment);
			DataSeries dsAdjusted = Close * reverseAdjustment;
			ChartPane adjPane = PricePane;
			if( Close.Description.Contains("Volume") )
			{
				dsAdjusted = Close / reverseAdjustment;
				adjPane = VolumePane;
			}
			dsAdjusted.Description = "Non-adjusted price";

			DataSeries InsiderBuyingSeriesOI = null;
			
			if(datasetFlag == "OI")
			{
				InsiderBuyingSeriesOI = new DataSeries( Bars, "Insider Buying (OI)" );
				InsiderBuyingSeriesOI.Description = "Insider Buying $ (OI)";
			}
			
			if(showDebug == true)
			{
				PrintDebug("################ Open Insider ################");
				PrintDebug("\n");
			}
			
			if(datasetFlag == "OI")
			{
				//Setup the very first tansaction bar number flag
				int currentTransBarFlag = -1;
			
				//Setup temp holding place for proceeds..
				double tempSumProceeds = 0;
			
				for (int bar = 1; bar < Bars.Count; bar++)
				{
					//double sumB = 0;
				
					foreach (FundamentalItem fiOI in fListOI)
					{
						if( fiOI.Bar == bar )
						{
							//sumB += fiOI.Value;
						
							//Only looking for buy information...
							
							if(fiOI.Value >= 0)
							{
								CultureInfo en = new CultureInfo("en-US");
								Thread.CurrentThread.CurrentCulture = en;
								String format = "d";
									
								//Try to get the bar # for the transaction date
								DateTime dtParsedTransactionDate = DateTime.ParseExact( fiOI.GetDetail("psdate"), format, en.DateTimeFormat );
								int transactionDateBarNumber = Bars.ConvertDateToBar( dtParsedTransactionDate, false );
									
								//Try to get the bar # for the reporting date
								DateTime dtParsedReportingDate = DateTime.ParseExact( fiOI.Date.ToShortDateString(), format, en.DateTimeFormat );
								int reportingDateBarNumber = Bars.ConvertDateToBar( dtParsedReportingDate, false );
																		
								if(currentTransBarFlag == bar){
									tempSumProceeds = tempSumProceeds + double.Parse(Regex.Replace(fiOI.GetDetail("proceeds"),"[^0-9.]+",""));
								} else if (currentTransBarFlag != bar)
								{	
									tempSumProceeds = double.Parse(Regex.Replace(fiOI.GetDetail("proceeds"),"[^0-9.]+",""));
									currentTransBarFlag = bar;
								}
									
								//Put the total proceeds into the InsiderBuyingSeriesOI Array located at bar
									
								InsiderBuyingSeriesOI[ bar ] = tempSumProceeds;
													
								if(showDebug == true)
								{
									PrintDebug("Total bought in $ Amount: $" + fiOI.GetDetail("proceeds"));
									PrintDebug("Number of Shares: " + fiOI.Value);
									PrintDebug("Price per Share: " + double.Parse(Regex.Replace(fiOI.GetDetail("proceeds"),"[^0-9.]+",""))/fiOI.Value);
									PrintDebug("Bars Count: " + Bars.Count);
									PrintDebug("Bar#: " + bar);
									PrintDebug("Reporting Date: " + fiOI.Date.ToShortDateString());
									//Reporting Date is at bar - 1								
									PrintDebug("Reporting Date Bar#: " + reportingDateBarNumber);
									PrintDebug("Transaction Date: " + fiOI.GetDetail("psdate"));
									PrintDebug("Parsed DateTime for Transaction Date: " + dtParsedTransactionDate);
									PrintDebug("Transaction Date Bar#: " + transactionDateBarNumber);
									PrintDebug("\n");
								}									
							}	
						}
					}
				}
			}
			
			DataSeries ibSum = null;
			
			if(datasetFlag == "OI")
			{			
				ibSum = Sum.Series( InsiderBuyingSeriesOI, 1 );
			}
			
			// Previous high
			DataSeries highest = Highest.Series( High, 250 );
			
			// Declined from the high by NN %
			DataSeries DeclineSeries = (100-paramDecline.Value)/100*highest;
			
			bool Setup = false;
			
			//Value of SetupBar
			int SetupBar = -1;
			
			int timeoutEntry = paramTimeoutEntry.ValueInt;
			double threshold = paramThreshold.Value;
			int daysToExit = paramExitAfter.ValueInt;
						
			for(int bar = 1; bar < Bars.Count; bar++)
			{
				//Check position and than sell it...
												
				if (IsLastPositionActive)
				{
					Position p = LastPosition;
					if ( bar+1 - p.EntryBar >= daysToExit )
						SellAtMarket( bar+1, p, "Timed" );
				}													
				else
				{
					if( !Setup )
					{
						//Check to see if the price is low enough...
						if( Low[bar] <= DeclineSeries[bar] )
						{
							Setup = true;
							SetupBar = bar;
						}
					}
					if( Setup )
					{
						if( ( bar + 1 - SetupBar ) < timeoutEntry )
						{
							if( ibSum[bar] >= threshold )
								if( BuyAtMarket( bar+1 ) != null )
								{
									//PrintDebug("Buy Date Here: " + Bars.Date[ bar+1 ].ToString("MM/dd/yyyy"));										
									Setup = false;
								}								
						} else
							// Reset if setup has timed out
							Setup = false;
					}
				}
			}
		}
	}
}